#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from __future__ import annotations

import logging
import os
import shutil
import sys
import time
from datetime import datetime, timedelta
from io import StringIO

import pytest
from freezegun import freeze_time

from babel import __version__ as VERSION
from babel.dates import format_datetime
from babel.messages import Catalog, frontend
from babel.messages.frontend import BaseError
from babel.messages.pofile import read_po, write_po
from babel.util import LOCALTZ
from tests.messages.consts import data_dir, get_po_file_path, i18n_dir


@pytest.fixture
def cli(monkeypatch, capsys):
    monkeypatch.chdir(data_dir)
    monkeypatch.setattr(sys, 'argv', ['pybabel'])
    _remove_log_handlers()
    yield frontend.CommandLineInterface()
    for dirname in ['lv_LV', 'ja_JP']:
        locale_dir = os.path.join(i18n_dir, dirname)
        if os.path.isdir(locale_dir):
            shutil.rmtree(locale_dir)
    _remove_log_handlers()


def _remove_log_handlers():
    # Logging handlers will be reused if possible (#227). This breaks the
    # implicit assumption that our newly created StringIO for sys.stderr
    # contains the console output. Removing the old handler ensures that a
    # new handler with our new StringIO instance will be used.
    log = logging.getLogger('babel')
    for handler in log.handlers:
        log.removeHandler(handler)


def test_usage(cli):
    with pytest.raises(SystemExit) as ei:
        cli.run(["pybabel"])
    assert ei.value.code == 2
    assert sys.stderr.getvalue().lower() == """\
usage: pybabel command [options] [args]

pybabel: error: no valid command or option passed. try the -h/--help option for more information.
"""


def test_list_locales(cli):
    """
    Test the command with the --list-locales arg.
    """
    result = cli.run(['pybabel', '--list-locales'])
    assert not result
    output = sys.stdout.getvalue()
    assert 'fr_CH' in output
    assert 'French (Switzerland)' in output
    assert "\nb'" not in output  # No bytes repr markers in output


def _run_init_catalog(cli):
    i18n_dir = os.path.join(data_dir, 'project', 'i18n')
    pot_path = os.path.join(data_dir, 'project', 'i18n', 'messages.pot')
    cli.run(['pybabel', 'init', '--locale', 'en_US', '-d', i18n_dir, '-i', pot_path])


def test_no_duplicated_output_for_multiple_runs(cli):
    _run_init_catalog(cli)
    first_output = sys.stderr.getvalue()
    _run_init_catalog(cli)
    second_output = sys.stderr.getvalue()[len(first_output):]

    # in case the log message is not duplicated we should get the same
    # output as before
    assert first_output == second_output


def test_frontend_can_log_to_predefined_handler(cli):
    custom_stream = StringIO()
    log = logging.getLogger('babel')
    log.addHandler(logging.StreamHandler(custom_stream))

    _run_init_catalog(cli)
    assert id(sys.stderr) != id(custom_stream)
    assert not sys.stderr.getvalue()
    assert custom_stream.getvalue()


def test_help(cli):
    with pytest.raises(SystemExit) as ei:
        cli.run(['pybabel', '--help'])
    assert not ei.value.code
    content = sys.stdout.getvalue().lower()
    assert 'options:' in content
    assert all(command in content for command in ('init', 'update', 'compile', 'extract'))


@freeze_time("1994-11-11")
def test_extract_with_default_mapping(cli, pot_file):
    cli.run([
        'pybabel',
        'extract',
        '--copyright-holder', 'FooBar, Inc.',
        '--project', 'TestProject', '--version', '0.1',
        '--msgid-bugs-address', 'bugs.address@email.tld',
        '-c', 'TRANSLATOR', '-c', 'TRANSLATORS:',
        '-o', pot_file, 'project',
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. TRANSLATOR: This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

#: project/ignored/this_wont_normally_be_here.py:11
msgid "FooBar"
msgid_plural "FooBars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


@freeze_time("1994-11-11")
def test_extract_with_mapping_file(cli, pot_file):
    cli.run([
        'pybabel',
        'extract',
        '--copyright-holder', 'FooBar, Inc.',
        '--project', 'TestProject', '--version', '0.1',
        '--msgid-bugs-address', 'bugs.address@email.tld',
        '--mapping', os.path.join(data_dir, 'mapping.cfg'),
        '-c', 'TRANSLATOR', '-c', 'TRANSLATORS:',
        '-o', pot_file, 'project',
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. TRANSLATOR: This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


@freeze_time("1994-11-11")
def test_extract_with_exact_file(cli, pot_file):
    """Tests that we can call extract with a particular file and only
    strings from that file get extracted. (Note the absence of strings from file1.py)
    """
    file_to_extract = os.path.join(data_dir, 'project', 'file2.py')
    cli.run([
        'pybabel',
        'extract',
        '--copyright-holder', 'FooBar, Inc.',
        '--project', 'TestProject', '--version', '0.1',
        '--msgid-bugs-address', 'bugs.address@email.tld',
        '--mapping', os.path.join(data_dir, 'mapping.cfg'),
        '-c', 'TRANSLATOR', '-c', 'TRANSLATORS:',
        '-o', pot_file, file_to_extract,
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


@freeze_time("1994-11-11")
def test_init_with_output_dir(cli):
    po_file = get_po_file_path('en_US')
    cli.run([
        'pybabel',
        'init',
        '--locale', 'en_US',
        '-d', os.path.join(i18n_dir),
        '-i', os.path.join(i18n_dir, 'messages.pot'),
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# English (United States) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: en_US\n"
"Language-Team: en_US <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    with open(po_file) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_init_singular_plural_forms(cli):
    po_file = get_po_file_path('ja_JP')
    cli.run([
        'pybabel',
        'init',
        '--locale', 'ja_JP',
        '-d', os.path.join(i18n_dir),
        '-i', os.path.join(i18n_dir, 'messages.pot'),
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Japanese (Japan) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: ja_JP\n"
"Language-Team: ja_JP <LL@li.org>\n"
"Plural-Forms: nplurals=1; plural=0;\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""

"""
    with open(po_file) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_init_more_than_2_plural_forms(cli):
    po_file = get_po_file_path('lv_LV')
    cli.run([
        'pybabel',
        'init',
        '--locale', 'lv_LV',
        '-d', i18n_dir,
        '-i', os.path.join(i18n_dir, 'messages.pot'),
    ])
    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Latvian (Latvia) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: lv_LV\n"
"Language-Team: lv_LV <LL@li.org>\n"
"Plural-Forms: nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 :"
" 2);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""

"""
    with open(po_file) as f:
        actual_content = f.read()
    assert expected_content == actual_content


def test_compile_catalog(cli):
    po_file = get_po_file_path('de_DE')
    mo_file = po_file.replace('.po', '.mo')
    cli.run(['pybabel', 'compile', '--locale', 'de_DE', '-d', i18n_dir])
    assert not os.path.isfile(mo_file), f'Expected no file at {mo_file!r}'
    assert sys.stderr.getvalue() == f'catalog {po_file} is marked as fuzzy, skipping\n'


def test_compile_fuzzy_catalog(cli):
    po_file = get_po_file_path('de_DE')
    mo_file = po_file.replace('.po', '.mo')
    try:
        cli.run(['pybabel', 'compile', '--locale', 'de_DE', '--use-fuzzy', '-d', i18n_dir])
        assert os.path.isfile(mo_file)
        assert sys.stderr.getvalue() == f'compiling catalog {po_file} to {mo_file}\n'
    finally:
        if os.path.isfile(mo_file):
            os.unlink(mo_file)


def test_compile_catalog_with_more_than_2_plural_forms(cli):
    po_file = get_po_file_path('ru_RU')
    mo_file = po_file.replace('.po', '.mo')
    try:
        cli.run(['pybabel', 'compile', '--locale', 'ru_RU', '--use-fuzzy', '-d', i18n_dir])
        assert os.path.isfile(mo_file)
        assert sys.stderr.getvalue() == f'compiling catalog {po_file} to {mo_file}\n'
    finally:
        if os.path.isfile(mo_file):
            os.unlink(mo_file)


def test_compile_catalog_multidomain(cli):
    po_foo = os.path.join(i18n_dir, 'de_DE', 'LC_MESSAGES', 'foo.po')
    po_bar = os.path.join(i18n_dir, 'de_DE', 'LC_MESSAGES', 'bar.po')
    mo_foo = po_foo.replace('.po', '.mo')
    mo_bar = po_bar.replace('.po', '.mo')
    try:
        cli.run([
            'pybabel', 'compile',
            '--locale', 'de_DE',
            '--domain', 'foo bar',
            '--use-fuzzy',
            '-d', i18n_dir,
        ])
        for mo_file in [mo_foo, mo_bar]:
            assert os.path.isfile(mo_file)
        assert sys.stderr.getvalue() == (
            f'compiling catalog {po_foo} to {mo_foo}\n'
            f'compiling catalog {po_bar} to {mo_bar}\n'
        )

    finally:
        for mo_file in [mo_foo, mo_bar]:
            if os.path.isfile(mo_file):
                os.unlink(mo_file)


def test_update(cli):
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = os.path.join(i18n_dir, 'temp-template.pot')
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)
    po_file = os.path.join(i18n_dir, 'temp1.po')
    cli.run(['pybabel', 'init', '-l', 'fi', '-o', po_file, '-i', tmpl_file])
    with open(po_file) as infp:
        catalog = read_po(infp)
        assert len(catalog) == 3

    # Add another entry to the template

    template.add("4")

    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    with open(po_file) as infp:
        catalog = read_po(infp)
        assert len(catalog) == 4  # Catalog was updated


def test_update_pot_creation_date(cli):
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = os.path.join(i18n_dir, 'temp-template.pot')
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)
    po_file = os.path.join(i18n_dir, 'temp1.po')
    cli.run(['pybabel', 'init', '-l', 'fi', '-o', po_file, '-i', tmpl_file])
    with open(po_file) as infp:
        catalog = read_po(infp)
        assert len(catalog) == 3
    original_catalog_creation_date = catalog.creation_date

    # Update the template creation date
    template.creation_date -= timedelta(minutes=3)
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    with open(po_file) as infp:
        catalog = read_po(infp)
        # We didn't ignore the creation date, so expect a diff
        assert catalog.creation_date != original_catalog_creation_date

    # Reset the "original"
    original_catalog_creation_date = catalog.creation_date

    # Update the template creation date again
    # This time, pass the ignore flag and expect the times are different
    template.creation_date -= timedelta(minutes=5)
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file, '--ignore-pot-creation-date'])

    with open(po_file) as infp:
        catalog = read_po(infp)
        # We ignored creation date, so it should not have changed
        assert catalog.creation_date == original_catalog_creation_date


def test_check(cli):
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = os.path.join(i18n_dir, 'temp-template.pot')
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)
    po_file = os.path.join(i18n_dir, 'temp1.po')
    cli.run(['pybabel', 'init', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Update the catalog file
    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Run a check without introducing any changes to the template
    cli.run(['pybabel', 'update', '--check', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Add a new entry and expect the check to fail
    template.add("4")
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    with pytest.raises(BaseError):
        cli.run(['pybabel', 'update', '--check', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Write the latest changes to the po-file
    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Update an entry and expect the check to fail
    template.add("4", locations=[("foo.py", 1)])
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    with pytest.raises(BaseError):
        cli.run(['pybabel', 'update', '--check', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])


def test_check_pot_creation_date(cli):
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = os.path.join(i18n_dir, 'temp-template.pot')
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)
    po_file = os.path.join(i18n_dir, 'temp1.po')
    cli.run(['pybabel', 'init', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Update the catalog file
    cli.run(['pybabel', 'update', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Run a check without introducing any changes to the template
    cli.run(['pybabel', 'update', '--check', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    # Run a check after changing the template creation date
    template.creation_date = datetime.now() - timedelta(minutes=5)
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    # Should fail without --ignore-pot-creation-date flag
    with pytest.raises(BaseError):
        cli.run(['pybabel', 'update', '--check', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])
    # Should pass with --ignore-pot-creation-date flag
    cli.run([
        'pybabel', 'update',
        '--check',
        '-l', 'fi_FI',
        '-o', po_file,
        '-i', tmpl_file,
        '--ignore-pot-creation-date',
    ])


def test_update_init_missing(cli):
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = os.path.join(i18n_dir, 'temp2-template.pot')
    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)
    po_file = os.path.join(i18n_dir, 'temp2.po')

    cli.run(['pybabel', 'update', '--init-missing', '-l', 'fi', '-o', po_file, '-i', tmpl_file])

    with open(po_file) as infp:
        catalog = read_po(infp)
        assert len(catalog) == 3

    # Add another entry to the template

    template.add("4")

    with open(tmpl_file, "wb") as outfp:
        write_po(outfp, template)

    cli.run(['pybabel', 'update', '--init-missing', '-l', 'fi_FI', '-o', po_file, '-i', tmpl_file])

    with open(po_file) as infp:
        catalog = read_po(infp)
        assert len(catalog) == 4  # Catalog was updated


def test_update_init_missing_creates_dest_dir(cli, tmp_path):
    template = Catalog()
    template.add("xyzzy")
    template.add("ferg")
    tmpl_file = tmp_path / 'temp.pot'
    with tmpl_file.open("wb") as outfp:
        write_po(outfp, template)

    dest_dir = tmp_path / 'newdir' / 'hierarchy'
    assert not dest_dir.exists()
    po_file = dest_dir / 'temp.po'

    cli.run(['pybabel', 'update', '--init-missing', '-l', 'ja', '-o', po_file, '-i', tmpl_file])
    assert dest_dir.exists()

    with po_file.open() as infp:
        assert len(read_po(infp)) == 2



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from datetime import datetime
from io import BytesIO, StringIO

import pytest

from babel import Locale
from babel.messages import Catalog, pofile
from babel.util import FixedOffsetTimezone


def test_preserve_locale():
    buf = StringIO(r'''msgid "foo"
msgstr "Voh"''')
    catalog = pofile.read_po(buf, locale='en_US')
    assert Locale('en', 'US') == catalog.locale


def test_locale_gets_overridden_by_file():
    buf = StringIO(r'''
msgid ""
msgstr ""
"Language: en_US\n"''')
    catalog = pofile.read_po(buf, locale='de')
    assert Locale('en', 'US') == catalog.locale
    buf = StringIO(r'''
msgid ""
msgstr ""
"Language: ko-KR\n"''')
    catalog = pofile.read_po(buf, locale='de')
    assert Locale('ko', 'KR') == catalog.locale


def test_preserve_domain():
    buf = StringIO(r'''msgid "foo"
msgstr "Voh"''')
    catalog = pofile.read_po(buf, domain='mydomain')
    assert catalog.domain == 'mydomain'


def test_applies_specified_encoding_during_read():
    buf = BytesIO('''
msgid ""
msgstr ""
"Project-Id-Version:  3.15\\n"
"Report-Msgid-Bugs-To: Fliegender Zirkus <fliegender@zirkus.de>\\n"
"POT-Creation-Date: 2007-09-27 11:19+0700\\n"
"PO-Revision-Date: 2007-09-27 21:42-0700\\n"
"Last-Translator: John <cleese@bavaria.de>\\n"
"Language-Team: German Lang <de@babel.org>\\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=iso-8859-1\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Generated-By: Babel 1.0dev-r313\\n"

msgid "foo"
msgstr "bär"'''.encode('iso-8859-1'))
    catalog = pofile.read_po(buf, locale='de_DE')
    assert catalog.get('foo').string == 'bär'


def test_encoding_header_read():
    buf = BytesIO(b'msgid ""\nmsgstr ""\n"Content-Type: text/plain; charset=mac_roman\\n"\n')
    catalog = pofile.read_po(buf, locale='xx_XX')
    assert catalog.charset == 'mac_roman'


def test_plural_forms_header_parsed():
    buf = BytesIO(b'msgid ""\nmsgstr ""\n"Plural-Forms: nplurals=42; plural=(n % 11);\\n"\n')
    catalog = pofile.read_po(buf, locale='xx_XX')
    assert catalog.plural_expr == '(n % 11)'
    assert catalog.num_plurals == 42


def test_read_multiline():
    buf = StringIO(r'''msgid ""
"Here's some text that\n"
"includesareallylongwordthatmightbutshouldnt"
" throw us into an infinite "
"loop\n"
msgstr ""''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 1
    message = list(catalog)[1]
    assert message.id == (
        "Here's some text that\nincludesareallylongwordthat"
        "mightbutshouldnt throw us into an infinite loop\n"
    )


def test_fuzzy_header():
    buf = StringIO(r'''
# Translations template for AReallyReallyLongNameForAProject.
# Copyright (C) 2007 ORGANIZATION
# This file is distributed under the same license as the
# AReallyReallyLongNameForAProject project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
#, fuzzy
''')
    catalog = pofile.read_po(buf)
    assert len(list(catalog)) == 1
    assert list(catalog)[0].fuzzy


def test_not_fuzzy_header():
    buf = StringIO(r'''
# Translations template for AReallyReallyLongNameForAProject.
# Copyright (C) 2007 ORGANIZATION
# This file is distributed under the same license as the
# AReallyReallyLongNameForAProject project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
''')
    catalog = pofile.read_po(buf)
    assert len(list(catalog)) == 1
    assert not list(catalog)[0].fuzzy


def test_header_entry():
    buf = StringIO(r'''
# SOME DESCRIPTIVE TITLE.
# Copyright (C) 2007 THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version:  3.15\n"
"Report-Msgid-Bugs-To: Fliegender Zirkus <fliegender@zirkus.de>\n"
"POT-Creation-Date: 2007-09-27 11:19+0700\n"
"PO-Revision-Date: 2007-09-27 21:42-0700\n"
"Last-Translator: John <cleese@bavaria.de>\n"
"Language: de\n"
"Language-Team: German Lang <de@babel.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=iso-8859-2\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel 1.0dev-r313\n"
''')
    catalog = pofile.read_po(buf)
    assert len(list(catalog)) == 1
    assert catalog.version == '3.15'
    assert catalog.msgid_bugs_address == 'Fliegender Zirkus <fliegender@zirkus.de>'
    assert datetime(2007, 9, 27, 11, 19, tzinfo=FixedOffsetTimezone(7 * 60)) == catalog.creation_date
    assert catalog.last_translator == 'John <cleese@bavaria.de>'
    assert Locale('de') == catalog.locale
    assert catalog.language_team == 'German Lang <de@babel.org>'
    assert catalog.charset == 'iso-8859-2'
    assert list(catalog)[0].fuzzy


def test_obsolete_message():
    buf = StringIO(r'''# This is an obsolete message
#~ msgid "foo"
#~ msgstr "Voh"

# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 1
    assert len(catalog.obsolete) == 1
    message = catalog.obsolete['foo']
    assert message.id == 'foo'
    assert message.string == 'Voh'
    assert message.user_comments == ['This is an obsolete message']


def test_obsolete_message_ignored():
    buf = StringIO(r'''# This is an obsolete message
#~ msgid "foo"
#~ msgstr "Voh"

# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf, ignore_obsolete=True)
    assert len(catalog) == 1
    assert len(catalog.obsolete) == 0


def test_multi_line_obsolete_message():
    buf = StringIO(r'''# This is an obsolete message
#~ msgid ""
#~ "foo"
#~ "foo"
#~ msgstr ""
#~ "Voh"
#~ "Vooooh"

# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog.obsolete) == 1
    message = catalog.obsolete['foofoo']
    assert message.id == 'foofoo'
    assert message.string == 'VohVooooh'
    assert message.user_comments == ['This is an obsolete message']


def test_unit_following_multi_line_obsolete_message():
    buf = StringIO(r'''# This is an obsolete message
#~ msgid ""
#~ "foo"
#~ "fooooooo"
#~ msgstr ""
#~ "Voh"
#~ "Vooooh"

# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 1
    message = catalog['bar']
    assert message.id == 'bar'
    assert message.string == 'Bahr'
    assert message.user_comments == ['This message is not obsolete']


def test_unit_before_obsolete_is_not_obsoleted():
    buf = StringIO(r'''
# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"

# This is an obsolete message
#~ msgid ""
#~ "foo"
#~ "fooooooo"
#~ msgstr ""
#~ "Voh"
#~ "Vooooh"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 1
    message = catalog['bar']
    assert message.id == 'bar'
    assert message.string == 'Bahr'
    assert message.user_comments == ['This message is not obsolete']


def test_with_context():
    buf = BytesIO(b'''# Some string in the menu
#: main.py:1
msgctxt "Menu"
msgid "foo"
msgstr "Voh"

# Another string in the menu
#: main.py:2
msgctxt "Menu"
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf, ignore_obsolete=True)
    assert len(catalog) == 2
    message = catalog.get('foo', context='Menu')
    assert message.context == 'Menu'
    message = catalog.get('bar', context='Menu')
    assert message.context == 'Menu'

    # And verify it pass through write_po
    out_buf = BytesIO()
    pofile.write_po(out_buf, catalog, omit_header=True)
    assert out_buf.getvalue().strip() == buf.getvalue().strip()


def test_obsolete_message_with_context():
    buf = StringIO('''
# This message is not obsolete
msgid "baz"
msgstr "Bazczch"

# This is an obsolete message
#~ msgctxt "other"
#~ msgid "foo"
#~ msgstr "Voh"

# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 2
    assert len(catalog.obsolete) == 1
    message = catalog.obsolete[("foo", "other")]
    assert message.context == 'other'
    assert message.string == 'Voh'


def test_obsolete_messages_with_context():
    buf = StringIO('''
# This is an obsolete message
#~ msgctxt "apple"
#~ msgid "foo"
#~ msgstr "Foo"

# This is an obsolete message with the same id but different context
#~ msgctxt "orange"
#~ msgid "foo"
#~ msgstr "Bar"
''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 0
    assert len(catalog.obsolete) == 2
    assert 'foo' not in catalog.obsolete

    apple_msg = catalog.obsolete[('foo', 'apple')]
    assert apple_msg.id == 'foo'
    assert apple_msg.string == 'Foo'
    assert apple_msg.user_comments == ['This is an obsolete message']

    orange_msg = catalog.obsolete[('foo', 'orange')]
    assert orange_msg.id == 'foo'
    assert orange_msg.string == 'Bar'
    assert orange_msg.user_comments == ['This is an obsolete message with the same id but different context']


def test_obsolete_messages_roundtrip():
    buf = StringIO('''\
# This message is not obsolete
#: main.py:1
msgid "bar"
msgstr "Bahr"

# This is an obsolete message
#~ msgid "foo"
#~ msgstr "Voh"

# This is an obsolete message
#~ msgctxt "apple"
#~ msgid "foo"
#~ msgstr "Foo"

# This is an obsolete message with the same id but different context
#~ msgctxt "orange"
#~ msgid "foo"
#~ msgstr "Bar"

''')
    generated_po_file = ''.join(pofile.generate_po(pofile.read_po(buf), omit_header=True))
    assert buf.getvalue() == generated_po_file


def test_multiline_context():
    buf = StringIO('''
msgctxt "a really long "
"message context "
"why?"
msgid "mid"
msgstr "mst"
    ''')
    catalog = pofile.read_po(buf)
    assert len(catalog) == 1
    message = catalog.get('mid', context="a really long message context why?")
    assert message is not None
    assert message.context == 'a really long message context why?'


def test_with_context_two():
    buf = BytesIO(b'''msgctxt "Menu"
msgid "foo"
msgstr "Voh"

msgctxt "Mannu"
msgid "bar"
msgstr "Bahr"
''')
    catalog = pofile.read_po(buf, ignore_obsolete=True)
    assert len(catalog) == 2
    message = catalog.get('foo', context='Menu')
    assert message.context == 'Menu'
    message = catalog.get('bar', context='Mannu')
    assert message.context == 'Mannu'

    # And verify it pass through write_po
    out_buf = BytesIO()
    pofile.write_po(out_buf, catalog, omit_header=True)
    assert out_buf.getvalue().strip() == buf.getvalue().strip(), out_buf.getvalue()


def test_single_plural_form():
    buf = StringIO(r'''msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh"''')
    catalog = pofile.read_po(buf, locale='ja_JP')
    assert len(catalog) == 1
    assert catalog.num_plurals == 1
    message = catalog['foo']
    assert len(message.string) == 1


def test_singular_plural_form():
    buf = StringIO(r'''msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh"
msgstr[1] "Vohs"''')
    catalog = pofile.read_po(buf, locale='nl_NL')
    assert len(catalog) == 1
    assert catalog.num_plurals == 2
    message = catalog['foo']
    assert len(message.string) == 2


def test_more_than_two_plural_forms():
    buf = StringIO(r'''msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh"
msgstr[1] "Vohs"
msgstr[2] "Vohss"''')
    catalog = pofile.read_po(buf, locale='lv_LV')
    assert len(catalog) == 1
    assert catalog.num_plurals == 3
    message = catalog['foo']
    assert len(message.string) == 3
    assert message.string[2] == 'Vohss'


def test_plural_with_square_brackets():
    buf = StringIO(r'''msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh [text]"
msgstr[1] "Vohs [text]"''')
    catalog = pofile.read_po(buf, locale='nb_NO')
    assert len(catalog) == 1
    assert catalog.num_plurals == 2
    message = catalog['foo']
    assert len(message.string) == 2


def test_obsolete_plural_with_square_brackets():
    buf = StringIO('''\
#~ msgid "foo"
#~ msgid_plural "foos"
#~ msgstr[0] "Voh [text]"
#~ msgstr[1] "Vohs [text]"
''')
    catalog = pofile.read_po(buf, locale='nb_NO')
    assert len(catalog) == 0
    assert len(catalog.obsolete) == 1
    assert catalog.num_plurals == 2
    message = catalog.obsolete['foo']
    assert len(message.string) == 2
    assert message.string[0] == 'Voh [text]'
    assert message.string[1] == 'Vohs [text]'


def test_missing_plural():
    buf = StringIO('''\
msgid ""
msgstr ""
"Plural-Forms: nplurals=3; plural=(n < 2) ? n : 2;\n"

msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh [text]"
msgstr[1] "Vohs [text]"
''')
    catalog = pofile.read_po(buf, locale='nb_NO')
    assert len(catalog) == 1
    assert catalog.num_plurals == 3
    message = catalog['foo']
    assert len(message.string) == 3
    assert message.string[0] == 'Voh [text]'
    assert message.string[1] == 'Vohs [text]'
    assert message.string[2] == ''


def test_missing_plural_in_the_middle():
    buf = StringIO('''\
msgid ""
msgstr ""
"Plural-Forms: nplurals=3; plural=(n < 2) ? n : 2;\n"

msgid "foo"
msgid_plural "foos"
msgstr[0] "Voh [text]"
msgstr[2] "Vohs [text]"
''')
    catalog = pofile.read_po(buf, locale='nb_NO')
    assert len(catalog) == 1
    assert catalog.num_plurals == 3
    message = catalog['foo']
    assert len(message.string) == 3
    assert message.string[0] == 'Voh [text]'
    assert message.string[1] == ''
    assert message.string[2] == 'Vohs [text]'


def test_with_location():
    buf = StringIO('''\
#: main.py:1 \u2068filename with whitespace.py\u2069:123
msgid "foo"
msgstr "bar"
''')
    catalog = pofile.read_po(buf, locale='de_DE')
    assert len(catalog) == 1
    message = catalog['foo']
    assert message.string == 'bar'
    assert message.locations == [("main.py", 1), ("filename with whitespace.py", 123)]


def test_abort_invalid_po_file():
    invalid_po = '''
        msgctxt ""
        "{\"checksum\": 2148532640, \"cxt\": \"collector_thankyou\", \"id\": "
        "270005359}"
        msgid ""
        "Thank you very much for your time.\n"
        "If you have any questions regarding this survey, please contact Fulano "
        "at nadie@blah.com"
        msgstr "Merci de prendre le temps de remplir le sondage.
        Pour toute question, veuillez communiquer avec Fulano  à nadie@blah.com
        "
    '''
    invalid_po_2 = '''
        msgctxt ""
        "{\"checksum\": 2148532640, \"cxt\": \"collector_thankyou\", \"id\": "
        "270005359}"
        msgid ""
        "Thank you very much for your time.\n"
        "If you have any questions regarding this survey, please contact Fulano "
        "at fulano@blah.com."
        msgstr "Merci de prendre le temps de remplir le sondage.
        Pour toute question, veuillez communiquer avec Fulano a fulano@blah.com
        "
        '''
    # Catalog not created, throws Unicode Error
    buf = StringIO(invalid_po)
    output = pofile.read_po(buf, locale='fr', abort_invalid=False)
    assert isinstance(output, Catalog)

    # Catalog not created, throws PoFileError
    buf = StringIO(invalid_po_2)
    with pytest.raises(pofile.PoFileError):
        pofile.read_po(buf, locale='fr', abort_invalid=True)

    # Catalog is created with warning, no abort
    buf = StringIO(invalid_po_2)
    output = pofile.read_po(buf, locale='fr', abort_invalid=False)
    assert isinstance(output, Catalog)

    # Catalog not created, aborted with PoFileError
    buf = StringIO(invalid_po_2)
    with pytest.raises(pofile.PoFileError):
        pofile.read_po(buf, locale='fr', abort_invalid=True)


def test_invalid_pofile_with_abort_flag():
    parser = pofile.PoFileParser(None, abort_invalid=True)
    lineno = 10
    line = 'Algo esta mal'
    msg = 'invalid file'
    with pytest.raises(pofile.PoFileError):
        parser._invalid_pofile(line, lineno, msg)



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import decimal

import pytest

from babel import numbers


def test_can_parse_decimals():
    assert decimal.Decimal('1099.98') == numbers.parse_decimal('1,099.98', locale='en_US')
    assert decimal.Decimal('1099.98') == numbers.parse_decimal('1.099,98', locale='de')
    assert decimal.Decimal('1099.98') == numbers.parse_decimal('1,099.98', locale='ar', numbering_system="default")
    assert decimal.Decimal('1099.98') == numbers.parse_decimal('1٬099٫98', locale='ar_EG', numbering_system="default")
    with pytest.raises(numbers.NumberFormatError):
        numbers.parse_decimal('2,109,998', locale='de')
    with pytest.raises(numbers.UnsupportedNumberingSystemError):
        numbers.parse_decimal('2,109,998', locale='de', numbering_system="unknown")


def test_parse_decimal_strict_mode():
    # Numbers with a misplaced grouping symbol should be rejected
    with pytest.raises(numbers.NumberFormatError) as info:
        numbers.parse_decimal('11.11', locale='de', strict=True)
    assert info.value.suggestions == ['1.111', '11,11']
    # Numbers with two misplaced grouping symbols should be rejected
    with pytest.raises(numbers.NumberFormatError) as info:
        numbers.parse_decimal('80.00.00', locale='de', strict=True)
    assert info.value.suggestions == ['800.000']
    # Partially grouped numbers should be rejected
    with pytest.raises(numbers.NumberFormatError) as info:
        numbers.parse_decimal('2000,000', locale='en_US', strict=True)
    assert info.value.suggestions == ['2,000,000', '2,000']
    # Numbers with duplicate grouping symbols should be rejected
    with pytest.raises(numbers.NumberFormatError) as info:
        numbers.parse_decimal('0,,000', locale='en_US', strict=True)
    assert info.value.suggestions == ['0']
    # Return only suggestion for 0 on strict
    with pytest.raises(numbers.NumberFormatError) as info:
        numbers.parse_decimal('0.00', locale='de', strict=True)
    assert info.value.suggestions == ['0']
    # Properly formatted numbers should be accepted
    assert str(numbers.parse_decimal('1.001', locale='de', strict=True)) == '1001'
    # Trailing zeroes should be accepted
    assert str(numbers.parse_decimal('3.00', locale='en_US', strict=True)) == '3.00'
    # Numbers with a grouping symbol and no trailing zeroes should be accepted
    assert str(numbers.parse_decimal('3,400.6', locale='en_US', strict=True)) == '3400.6'
    # Numbers with a grouping symbol and trailing zeroes (not all zeroes after decimal) should be accepted
    assert str(numbers.parse_decimal('3,400.60', locale='en_US', strict=True)) == '3400.60'
    # Numbers with a grouping symbol and trailing zeroes (all zeroes after decimal) should be accepted
    assert str(numbers.parse_decimal('3,400.00', locale='en_US', strict=True)) == '3400.00'
    assert str(numbers.parse_decimal('3,400.0000', locale='en_US', strict=True)) == '3400.0000'
    # Numbers with a grouping symbol and no decimal part should be accepted
    assert str(numbers.parse_decimal('3,800', locale='en_US', strict=True)) == '3800'
    # Numbers without any grouping symbol should be accepted
    assert str(numbers.parse_decimal('2000.1', locale='en_US', strict=True)) == '2000.1'
    # Numbers without any grouping symbol and no decimal should be accepted
    assert str(numbers.parse_decimal('2580', locale='en_US', strict=True)) == '2580'
    # High precision numbers should be accepted
    assert str(numbers.parse_decimal('5,000001', locale='fr', strict=True)) == '5.000001'



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from io import BytesIO, StringIO

import pytest

from babel.core import Locale
from babel.messages import pofile
from babel.messages.catalog import Catalog
from babel.messages.pofile import _enclose_filename_if_necessary, _extract_locations


def test_enclosed_filenames_in_location_comment():
    catalog = Catalog()
    catalog.add("foo", lineno=2, locations=[("main 1.py", 1)], string="")
    catalog.add("bar", lineno=6, locations=[("other.py", 2)], string="")
    catalog.add("baz", lineno=10, locations=[("main 1.py", 3), ("other.py", 4)], string="")
    buf = BytesIO()
    pofile.write_po(buf, catalog, omit_header=True, include_lineno=True)
    buf.seek(0)
    catalog2 = pofile.read_po(buf)
    assert True is catalog.is_identical(catalog2)


def test_unescape():
    escaped = '"Say:\\n  \\"hello, world!\\"\\n"'
    unescaped = 'Say:\n  "hello, world!"\n'
    assert unescaped != escaped
    assert unescaped == pofile.unescape(escaped)


def test_unescape_of_quoted_newline():
    # regression test for #198
    assert pofile.unescape(r'"\\n"') == '\\n'


def test_denormalize_on_msgstr_without_empty_first_line():
    # handle irregular multi-line msgstr (no "" as first line)
    # gracefully (#171)
    msgstr = '"multi-line\\n"\n" translation"'
    expected_denormalized = 'multi-line\n translation'

    assert expected_denormalized == pofile.denormalize(msgstr)
    assert expected_denormalized == pofile.denormalize(f'""\n{msgstr}')


@pytest.mark.parametrize(("line", "locations"), [
    ("\u2068file1.po\u2069", ["file1.po"]),
    ("file1.po \u2068file 2.po\u2069 file3.po", ["file1.po", "file 2.po", "file3.po"]),
    ("file1.po:1 \u2068file 2.po\u2069:2 file3.po:3", ["file1.po:1", "file 2.po:2", "file3.po:3"]),
    ("\u2068file1.po\u2069:1 \u2068file\t2.po\u2069:2 file3.po:3",
     ["file1.po:1", "file\t2.po:2", "file3.po:3"]),
    ("file1.po  file2.po", ["file1.po", "file2.po"]),
    ("file1.po \u2068\u2069 file2.po", ["file1.po", "file2.po"]),
])
def test_extract_locations_valid_location_comment(line, locations):
    assert locations == _extract_locations(line)


@pytest.mark.parametrize(("line",), [
    ("\u2068file 1.po",),
    ("file 1.po\u2069",),
    ("\u2069file 1.po\u2068",),
    ("\u2068file 1.po:1 \u2068file 2.po\u2069:2",),
    ("\u2068file 1.po\u2069:1 file 2.po\u2069:2",),
])
def test_extract_locations_invalid_location_comment(line):
    with pytest.raises(ValueError):
        _extract_locations(line)


@pytest.mark.parametrize(("filename",), [
    ("file.po",),
    ("file_a.po",),
    ("file-a.po",),
    ("file\n.po",),
    ("\u2068file.po\u2069",),
    ("\u2068file a.po\u2069",),
])
def test_enclose_filename_if_necessary_no_change(filename):
    assert filename == _enclose_filename_if_necessary(filename)


@pytest.mark.parametrize(("filename",), [
    ("file a.po",),
    ("file\ta.po",),
])
def test_enclose_filename_if_necessary_enclosed(filename):
    assert "\u2068" + filename + "\u2069" == _enclose_filename_if_necessary(filename)


def test_unknown_language_roundtrip():
    buf = StringIO(r'''
msgid ""
msgstr ""
"Language: sr_SP\n"''')
    catalog = pofile.read_po(buf)
    assert catalog.locale_identifier == 'sr_SP'
    assert not catalog.locale
    buf = BytesIO()
    pofile.write_po(buf, catalog)
    assert 'sr_SP' in buf.getvalue().decode()


def test_unknown_language_write():
    catalog = Catalog(locale='sr_SP')
    assert catalog.locale_identifier == 'sr_SP'
    assert not catalog.locale
    buf = BytesIO()
    pofile.write_po(buf, catalog)
    assert 'sr_SP' in buf.getvalue().decode()


def test_iterable_of_strings():
    """
    Test we can parse from an iterable of strings.
    """
    catalog = pofile.read_po(['msgid "foo"', 'msgstr "Voh"'], locale="en_US")
    assert catalog.locale == Locale("en", "US")
    assert catalog.get("foo").string == "Voh"


@pytest.mark.parametrize("order", [1, -1])
def test_iterable_of_mismatching_strings(order):
    # Mixing and matching byteses and strs in the same read_po call is not allowed.
    with pytest.raises(Exception):  # noqa: B017 (will raise either TypeError or AttributeError)
        pofile.read_po(['msgid "foo"', b'msgstr "Voh"'][::order])


def test_issue_1087():
    buf = StringIO(r'''
msgid ""
msgstr ""
"Language: \n"
''')
    assert pofile.read_po(buf).locale is None


@pytest.mark.parametrize("case", ['msgid "foo"', 'msgid "foo"\nmsgid_plural "foos"'])
@pytest.mark.parametrize("abort_invalid", [False, True])
def test_issue_1134(case: str, abort_invalid: bool):
    buf = StringIO(case)

    if abort_invalid:
        # Catalog not created, aborted with PoFileError
        with pytest.raises(pofile.PoFileError, match="missing msgstr for msgid 'foo' on 0"):
            pofile.read_po(buf, abort_invalid=True)
    else:
        # Catalog is created with warning, no abort
        output = pofile.read_po(buf)
        assert len(output) == 1
        assert output["foo"].string in ((''), ('', ''))



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from __future__ import annotations

import pytest

from babel.messages import frontend
from babel.messages.frontend import OptionError
from tests.messages.consts import TEST_PROJECT_DISTRIBUTION_DATA, data_dir
from tests.messages.utils import Distribution


@pytest.fixture
def compile_catalog_cmd(monkeypatch):
    monkeypatch.chdir(data_dir)
    dist = Distribution(TEST_PROJECT_DISTRIBUTION_DATA)
    cmd = frontend.CompileCatalog(dist)
    cmd.initialize_options()
    return cmd


def test_no_directory_or_output_file_specified(compile_catalog_cmd):
    compile_catalog_cmd.locale = 'en_US'
    compile_catalog_cmd.input_file = 'dummy'
    with pytest.raises(OptionError):
        compile_catalog_cmd.finalize_options()


def test_no_directory_or_input_file_specified(compile_catalog_cmd):
    compile_catalog_cmd.locale = 'en_US'
    compile_catalog_cmd.output_file = 'dummy'
    with pytest.raises(OptionError):
        compile_catalog_cmd.finalize_options()



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import sys
from io import BytesIO, StringIO

import pytest

from babel.messages import extract


def test_invalid_filter():
    buf = BytesIO(b"""\
msg1 = _(i18n_arg.replace(r'\"', '"'))
msg2 = ungettext(i18n_arg.replace(r'\"', '"'), multi_arg.replace(r'\"', '"'), 2)
msg3 = ungettext("Babel", multi_arg.replace(r'\"', '"'), 2)
msg4 = ungettext(i18n_arg.replace(r'\"', '"'), "Babels", 2)
msg5 = ungettext('bunny', 'bunnies', random.randint(1, 2))
msg6 = ungettext(arg0, 'bunnies', random.randint(1, 2))
msg7 = _(hello.there)
msg8 = gettext('Rabbit')
msg9 = dgettext('wiki', model.addPage())
msg10 = dngettext(domain, 'Page', 'Pages', 3)
""")
    messages = \
        list(extract.extract('python', buf, extract.DEFAULT_KEYWORDS, [],
                             {}))
    assert messages == [
        (5, ('bunny', 'bunnies'), [], None),
        (8, 'Rabbit', [], None),
        (10, ('Page', 'Pages'), [], None),
    ]


def test_invalid_extract_method():
    buf = BytesIO(b'')
    with pytest.raises(ValueError):
        list(extract.extract('spam', buf))


def test_different_signatures():
    buf = BytesIO(b"""
foo = _('foo', 'bar')
n = ngettext('hello', 'there', n=3)
n = ngettext(n=3, 'hello', 'there')
n = ngettext(n=3, *messages)
n = ngettext()
n = ngettext('foo')
""")
    messages = \
        list(extract.extract('python', buf, extract.DEFAULT_KEYWORDS, [],
                             {}))
    assert len(messages) == 2
    assert messages[0][1] == 'foo'
    assert messages[1][1] == ('hello', 'there')


def test_empty_string_msgid():
    buf = BytesIO(b"""\
msg = _('')
""")
    stderr = sys.stderr
    sys.stderr = StringIO()
    try:
        messages = \
            list(extract.extract('python', buf, extract.DEFAULT_KEYWORDS,
                                 [], {}))
        assert messages == []
        assert 'warning: Empty msgid.' in sys.stderr.getvalue()
    finally:
        sys.stderr = stderr


def test_warn_if_empty_string_msgid_found_in_context_aware_extraction_method():
    buf = BytesIO(b"\nmsg = pgettext('ctxt', '')\n")
    stderr = sys.stderr
    sys.stderr = StringIO()
    try:
        messages = extract.extract('python', buf)
        assert list(messages) == []
        assert 'warning: Empty msgid.' in sys.stderr.getvalue()
    finally:
        sys.stderr = stderr


def test_extract_allows_callable():
    def arbitrary_extractor(fileobj, keywords, comment_tags, options):
        return [(1, None, (), ())]
    for x in extract.extract(arbitrary_extractor, BytesIO(b"")):
        assert x[0] == 1


def test_future():
    buf = BytesIO(br"""
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
nbsp = _('\xa0')
""")
    messages = list(extract.extract('python', buf,
                                    extract.DEFAULT_KEYWORDS, [], {}))
    assert messages[0][1] == '\xa0'


def test_f_strings():
    buf = BytesIO(br"""
t1 = _('foobar')
t2 = _(f'spameggs' f'feast')  # should be extracted; constant parts only
t2 = _(f'spameggs' 'kerroshampurilainen')  # should be extracted (mixing f with no f)
t3 = _(f'''whoa! a '''  # should be extracted (continues on following lines)
f'flying shark'
'... hello'
)
t4 = _(f'spameggs {t1}')  # should not be extracted
""")
    messages = list(extract.extract('python', buf, extract.DEFAULT_KEYWORDS, [], {}))
    assert len(messages) == 4
    assert messages[0][1] == 'foobar'
    assert messages[1][1] == 'spameggsfeast'
    assert messages[2][1] == 'spameggskerroshampurilainen'
    assert messages[3][1] == 'whoa! a flying shark... hello'


def test_f_strings_non_utf8():
    buf = BytesIO(b"""
# -- coding: latin-1 --
t2 = _(f'\xe5\xe4\xf6' f'\xc5\xc4\xd6')
""")
    messages = list(extract.extract('python', buf, extract.DEFAULT_KEYWORDS, [], {}))
    assert len(messages) == 1
    assert messages[0][1] == 'åäöÅÄÖ'


def test_issue_1195():
    buf = BytesIO(b"""
foof = {
    'test_string': StringWithMeta(
        # NOTE: Text describing a test string
        string=_(
            'Text string that is on a new line'
        ),
    ),
}
""")
    messages = list(extract.extract('python', buf, {'_': None}, ["NOTE"], {}))
    message = messages[0]
    assert message[0] in (5, 6)  # Depends on whether #1126 is in
    assert message[1] == 'Text string that is on a new line'
    assert message[2] == ['NOTE: Text describing a test string']


def test_issue_1195_2():
    buf = BytesIO(b"""
# NOTE: This should still be considered, even if
#       the text is far away
foof = _(









            'Hey! Down here!')
""")
    messages = list(extract.extract('python', buf, {'_': None}, ["NOTE"], {}))
    message = messages[0]
    assert message[1] == 'Hey! Down here!'
    assert message[2] == [
        'NOTE: This should still be considered, even if',
        'the text is far away',
    ]



from __future__ import annotations

CUSTOM_EXTRACTOR_COOKIE = "custom extractor was here"


def custom_extractor(fileobj, keywords, comment_tags, options):
    if "treat" not in options:
        raise RuntimeError(f"The custom extractor refuses to run without a delicious treat; got {options!r}")
    return [(1, next(iter(keywords)), (CUSTOM_EXTRACTOR_COOKIE,), [])]


class Distribution:  # subset of distutils.dist.Distribution
    def __init__(self, attrs: dict) -> None:
        self.attrs = attrs

    def get_name(self) -> str:
        return self.attrs['name']

    def get_version(self) -> str:
        return self.attrs['version']

    @property
    def packages(self) -> list[str]:
        return self.attrs['packages']



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import codecs
from io import BytesIO

import pytest

from babel.messages import extract


def test_nested_calls():
    buf = BytesIO(b"""\
msg1 = _(i18n_arg.replace(r'\"', '"'))
msg2 = ungettext(i18n_arg.replace(r'\"', '"'), multi_arg.replace(r'\"', '"'), 2)
msg3 = ungettext("Babel", multi_arg.replace(r'\"', '"'), 2)
msg4 = ungettext(i18n_arg.replace(r'\"', '"'), "Babels", 2)
msg5 = ungettext('bunny', 'bunnies', random.randint(1, 2))
msg6 = ungettext(arg0, 'bunnies', random.randint(1, 2))
msg7 = _(hello.there)
msg8 = gettext('Rabbit')
msg9 = dgettext('wiki', model.addPage())
msg10 = dngettext(getDomain(), 'Page', 'Pages', 3)
msg11 = ngettext(
"bunny",
"bunnies",
len(bunnies)
)
""")
    messages = list(extract.extract_python(buf,
                                           extract.DEFAULT_KEYWORDS.keys(),
                                           [], {}))
    assert messages == [
        (1, '_', None, []),
        (2, 'ungettext', (None, None, None), []),
        (3, 'ungettext', ('Babel', None, None), []),
        (4, 'ungettext', (None, 'Babels', None), []),
        (5, 'ungettext', ('bunny', 'bunnies', None), []),
        (6, 'ungettext', (None, 'bunnies', None), []),
        (7, '_', None, []),
        (8, 'gettext', 'Rabbit', []),
        (9, 'dgettext', ('wiki', None), []),
        (10, 'dngettext', (None, 'Page', 'Pages', None), []),
        (12, 'ngettext', ('bunny', 'bunnies', None), []),
    ]


def test_extract_default_encoding_ascii():
    buf = BytesIO(b'_("a")')
    messages = list(extract.extract_python(
        buf, list(extract.DEFAULT_KEYWORDS), [], {},
    ))
    # Should work great in both py2 and py3
    assert messages == [(1, '_', 'a', [])]


def test_extract_default_encoding_utf8():
    buf = BytesIO('_("☃")'.encode('UTF-8'))
    messages = list(extract.extract_python(
        buf, list(extract.DEFAULT_KEYWORDS), [], {},
    ))
    assert messages == [(1, '_', '☃', [])]


def test_nested_comments():
    buf = BytesIO(b"""\
msg = ngettext('pylon',  # TRANSLATORS: shouldn't be
           'pylons', # TRANSLATORS: seeing this
           count)
""")
    messages = list(extract.extract_python(buf, ('ngettext',),
                                           ['TRANSLATORS:'], {}))
    assert messages == [(1, 'ngettext', ('pylon', 'pylons', None), [])]


def test_comments_with_calls_that_spawn_multiple_lines():
    buf = BytesIO(b"""\
# NOTE: This Comment SHOULD Be Extracted
add_notice(req, ngettext("Catalog deleted.",
                     "Catalogs deleted.", len(selected)))

# NOTE: This Comment SHOULD Be Extracted
add_notice(req, _("Locale deleted."))


# NOTE: This Comment SHOULD Be Extracted
add_notice(req, ngettext("Foo deleted.", "Foos deleted.", len(selected)))

# NOTE: This Comment SHOULD Be Extracted
# NOTE: And This One Too
add_notice(req, ngettext("Bar deleted.",
                     "Bars deleted.", len(selected)))
""")
    messages = list(extract.extract_python(buf, ('ngettext', '_'), ['NOTE:'],

                                           {'strip_comment_tags': False}))
    assert messages[0] == (2, 'ngettext', ('Catalog deleted.', 'Catalogs deleted.', None), ['NOTE: This Comment SHOULD Be Extracted'])
    assert messages[1] == (6, '_', 'Locale deleted.', ['NOTE: This Comment SHOULD Be Extracted'])
    assert messages[2] == (10, 'ngettext', ('Foo deleted.', 'Foos deleted.', None), ['NOTE: This Comment SHOULD Be Extracted'])
    assert messages[3] == (14, 'ngettext', ('Bar deleted.', 'Bars deleted.', None), ['NOTE: This Comment SHOULD Be Extracted', 'NOTE: And This One Too'])


def test_declarations():
    buf = BytesIO(b"""\
class gettext(object):
pass
def render_body(context,x,y=_('Page arg 1'),z=_('Page arg 2'),**pageargs):
pass
def ngettext(y='arg 1',z='arg 2',**pageargs):
pass
class Meta:
verbose_name = _('log entry')
""")
    messages = list(extract.extract_python(buf,
                                           extract.DEFAULT_KEYWORDS.keys(),
                                           [], {}))
    assert messages == [
        (3, '_', 'Page arg 1', []),
        (3, '_', 'Page arg 2', []),
        (8, '_', 'log entry', []),
    ]


def test_multiline():
    buf = BytesIO(b"""\
msg1 = ngettext('pylon',
            'pylons', count)
msg2 = ngettext('elvis',
            'elvises',
             count)
""")
    messages = list(extract.extract_python(buf, ('ngettext',), [], {}))
    assert messages == [
        (1, 'ngettext', ('pylon', 'pylons', None), []),
        (3, 'ngettext', ('elvis', 'elvises', None), []),
    ]


def test_dpgettext():
    buf = BytesIO(b"""\
msg1 = dpgettext('dev', 'Strings',
             'pylon')
msg2 = dpgettext('dev', 'Strings', 'elvis')
""")
    messages = list(extract.extract_python(buf, ('dpgettext',), [], {}))
    assert messages == [
        (1, 'dpgettext', ('dev', 'Strings', 'pylon'), []),
        (3, 'dpgettext', ('dev', 'Strings', 'elvis'), []),
    ]
    buf = BytesIO(b"""\
msg = dpgettext('dev', 'Strings', 'pylon',  # TRANSLATORS: shouldn't be
            )                # TRANSLATORS: seeing this
""")
    messages = list(extract.extract_python(buf, ('dpgettext',),['TRANSLATORS:'], {}))
    assert messages == [
        (1, 'dpgettext', ('dev', 'Strings', 'pylon', None), []),
    ]


def test_npgettext():
    buf = BytesIO(b"""\
msg1 = npgettext('Strings','pylon',
            'pylons', count)
msg2 = npgettext('Strings','elvis',
            'elvises',
             count)
""")
    messages = list(extract.extract_python(buf, ('npgettext',), [], {}))
    assert messages == [
        (1, 'npgettext', ('Strings', 'pylon', 'pylons', None), []),
        (3, 'npgettext', ('Strings', 'elvis', 'elvises', None), []),
    ]
    buf = BytesIO(b"""\
msg = npgettext('Strings', 'pylon',  # TRANSLATORS: shouldn't be
            'pylons', # TRANSLATORS: seeing this
            count)
""")
    messages = list(extract.extract_python(buf, ('npgettext',),
                                           ['TRANSLATORS:'], {}))
    assert messages == [
        (1, 'npgettext', ('Strings', 'pylon', 'pylons', None), []),
    ]


def test_dnpgettext():
    buf = BytesIO(b"""\
msg1 = dnpgettext('dev', 'Strings','pylon',
            'pylons', count)
msg2 = dnpgettext('dev', 'Strings','elvis',
            'elvises',
             count)
""")
    messages = list(extract.extract_python(buf, ('dnpgettext',), [], {}))
    assert messages == [
        (1, 'dnpgettext', ('dev', 'Strings', 'pylon', 'pylons', None), []),
        (3, 'dnpgettext', ('dev', 'Strings', 'elvis', 'elvises', None), []),
    ]
    buf = BytesIO(b"""\
msg = dnpgettext('dev', 'Strings', 'pylon',  # TRANSLATORS: shouldn't be
             'pylons', # TRANSLATORS: seeing this
             count)
""")
    messages = list(extract.extract_python(buf, ('dnpgettext',),['TRANSLATORS:'], {}))
    assert messages == [
        (1, 'dnpgettext', ('dev', 'Strings', 'pylon', 'pylons', None), []),
    ]


def test_triple_quoted_strings():
    buf = BytesIO(b"""\
msg1 = _('''pylons''')
msg2 = ngettext(r'''elvis''', \"\"\"elvises\"\"\", count)
msg2 = ngettext(\"\"\"elvis\"\"\", 'elvises', count)
""")
    messages = list(extract.extract_python(buf,
                                           extract.DEFAULT_KEYWORDS.keys(),
                                           [], {}))
    assert messages == [
        (1, '_', 'pylons', []),
        (2, 'ngettext', ('elvis', 'elvises', None), []),
        (3, 'ngettext', ('elvis', 'elvises', None), []),
    ]


def test_multiline_strings():
    buf = BytesIO(b"""\
_('''This module provides internationalization and localization
support for your Python programs by providing an interface to the GNU
gettext message catalog library.''')
""")
    messages = list(extract.extract_python(buf,
                                           extract.DEFAULT_KEYWORDS.keys(),
                                           [], {}))
    assert messages == [
        (1, '_',
        'This module provides internationalization and localization\n'
        'support for your Python programs by providing an interface to '
        'the GNU\ngettext message catalog library.', []),
    ]


def test_concatenated_strings():
    buf = BytesIO(b"""\
foobar = _('foo' 'bar')
""")
    messages = list(extract.extract_python(buf,
                                           extract.DEFAULT_KEYWORDS.keys(),
                                           [], {}))
    assert messages[0][2] == 'foobar'


def test_unicode_string_arg():
    buf = BytesIO(b"msg = _('Foo Bar')")
    messages = list(extract.extract_python(buf, ('_',), [], {}))
    assert messages[0][2] == 'Foo Bar'


def test_comment_tag():
    buf = BytesIO(b"""
# NOTE: A translation comment
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == ['NOTE: A translation comment']


def test_comment_tag_multiline():
    buf = BytesIO(b"""
# NOTE: A translation comment
# with a second line
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == ['NOTE: A translation comment', 'with a second line']


def test_translator_comments_with_previous_non_translator_comments():
    buf = BytesIO(b"""
# This shouldn't be in the output
# because it didn't start with a comment tag
# NOTE: A translation comment
# with a second line
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == ['NOTE: A translation comment', 'with a second line']


def test_comment_tags_not_on_start_of_comment():
    buf = BytesIO(b"""
# This shouldn't be in the output
# because it didn't start with a comment tag
# do NOTE: this will not be a translation comment
# NOTE: This one will be
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == ['NOTE: This one will be']


def test_multiple_comment_tags():
    buf = BytesIO(b"""
# NOTE1: A translation comment for tag1
# with a second line
msg = _('Foo Bar1')

# NOTE2: A translation comment for tag2
msg = _('Foo Bar2')
""")
    messages = list(extract.extract_python(buf, ('_',),
                                           ['NOTE1:', 'NOTE2:'], {}))
    assert messages[0][2] == 'Foo Bar1'
    assert messages[0][3] == ['NOTE1: A translation comment for tag1', 'with a second line']
    assert messages[1][2] == 'Foo Bar2'
    assert messages[1][3] == ['NOTE2: A translation comment for tag2']


def test_two_succeeding_comments():
    buf = BytesIO(b"""
# NOTE: one
# NOTE: two
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == ['NOTE: one', 'NOTE: two']


def test_invalid_translator_comments():
    buf = BytesIO(b"""
# NOTE: this shouldn't apply to any messages
hello = 'there'

msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == []


def test_invalid_translator_comments2():
    buf = BytesIO(b"""
# NOTE: Hi!
hithere = _('Hi there!')

# NOTE: you should not be seeing this in the .po
rows = [[v for v in range(0,10)] for row in range(0,10)]

# this (NOTE:) should not show up either
hello = _('Hello')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Hi there!'
    assert messages[0][3] == ['NOTE: Hi!']
    assert messages[1][2] == 'Hello'
    assert messages[1][3] == []


def test_invalid_translator_comments3():
    buf = BytesIO(b"""
# NOTE: Hi,

# there!
hithere = _('Hi there!')
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Hi there!'
    assert messages[0][3] == []


def test_comment_tag_with_leading_space():
    buf = BytesIO(b"""
#: A translation comment
#: with leading spaces
msg = _('Foo Bar')
""")
    messages = list(extract.extract_python(buf, ('_',), [':'], {}))
    assert messages[0][2] == 'Foo Bar'
    assert messages[0][3] == [': A translation comment', ': with leading spaces']


def test_different_signatures():
    buf = BytesIO(b"""
foo = _('foo', 'bar')
n = ngettext('hello', 'there', n=3)
n = ngettext(n=3, 'hello', 'there')
n = ngettext(n=3, *messages)
n = ngettext()
n = ngettext('foo')
""")
    messages = list(extract.extract_python(buf, ('_', 'ngettext'), [], {}))
    assert messages[0][2] == ('foo', 'bar')
    assert messages[1][2] == ('hello', 'there', None)
    assert messages[2][2] == (None, 'hello', 'there')
    assert messages[3][2] == (None, None)
    assert messages[4][2] is None
    assert messages[5][2] == 'foo'


def test_utf8_message():
    buf = BytesIO("""
# NOTE: hello
msg = _('Bonjour à tous')
""".encode('utf-8'))
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'],
                                           {'encoding': 'utf-8'}))
    assert messages[0][2] == 'Bonjour à tous'
    assert messages[0][3] == ['NOTE: hello']


def test_utf8_message_with_magic_comment():
    buf = BytesIO("""# -*- coding: utf-8 -*-
# NOTE: hello
msg = _('Bonjour à tous')
""".encode('utf-8'))
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Bonjour à tous'
    assert messages[0][3] == ['NOTE: hello']


def test_utf8_message_with_utf8_bom():
    buf = BytesIO(codecs.BOM_UTF8 + """
# NOTE: hello
msg = _('Bonjour à tous')
""".encode('utf-8'))
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Bonjour à tous'
    assert messages[0][3] == ['NOTE: hello']


def test_utf8_message_with_utf8_bom_and_magic_comment():
    buf = BytesIO(codecs.BOM_UTF8 + """# -*- coding: utf-8 -*-
# NOTE: hello
msg = _('Bonjour à tous')
""".encode('utf-8'))
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Bonjour à tous'
    assert messages[0][3] == ['NOTE: hello']


def test_utf8_bom_with_latin_magic_comment_fails():
    buf = BytesIO(codecs.BOM_UTF8 + """# -*- coding: latin-1 -*-
# NOTE: hello
msg = _('Bonjour à tous')
""".encode('utf-8'))
    with pytest.raises(SyntaxError):
        list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))


def test_utf8_raw_strings_match_unicode_strings():
    buf = BytesIO(codecs.BOM_UTF8 + """
msg = _('Bonjour à tous')
msgu = _('Bonjour à tous')
""".encode('utf-8'))
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == 'Bonjour à tous'
    assert messages[0][2] == messages[1][2]


def test_extract_strip_comment_tags():
    buf = BytesIO(b"""\
#: This is a comment with a very simple
#: prefix specified
_('Servus')

# NOTE: This is a multiline comment with
# a prefix too
_('Babatschi')""")
    messages = list(extract.extract('python', buf, comment_tags=['NOTE:', ':'],
                                    strip_comment_tags=True))
    assert messages[0][1] == 'Servus'
    assert messages[0][2] == ['This is a comment with a very simple', 'prefix specified']
    assert messages[1][1] == 'Babatschi'
    assert messages[1][2] == ['This is a multiline comment with', 'a prefix too']


def test_nested_messages():
    buf = BytesIO(b"""
# NOTE: First
_('Hello, {name}!', name=_('Foo Bar'))

# NOTE: Second
_('Hello, {name1} and {name2}!', name1=_('Heungsub'),
name2=_('Armin'))

# NOTE: Third
_('Hello, {0} and {1}!', _('Heungsub'),
_('Armin'))
""")
    messages = list(extract.extract_python(buf, ('_',), ['NOTE:'], {}))
    assert messages[0][2] == ('Hello, {name}!', None)
    assert messages[0][3] == ['NOTE: First']
    assert messages[1][2] == 'Foo Bar'
    assert messages[1][3] == []
    assert messages[2][2] == ('Hello, {name1} and {name2}!', None)
    assert messages[2][3] == ['NOTE: Second']
    assert messages[3][2] == 'Heungsub'
    assert messages[3][3] == []
    assert messages[4][2] == 'Armin'
    assert messages[4][3] == []
    assert messages[5][2] == ('Hello, {0} and {1}!', None)
    assert messages[5][3] == ['NOTE: Third']
    assert messages[6][2] == 'Heungsub'
    assert messages[6][3] == []
    assert messages[7][2] == 'Armin'
    assert messages[7][3] == []



from __future__ import annotations

import pathlib
from io import BytesIO

import pytest

from babel.messages import frontend

toml_test_cases_path = pathlib.Path(__file__).parent / "toml-test-cases"
assert toml_test_cases_path.is_dir(), "toml-test-cases directory not found"


def parse_toml(cfg: bytes | str):
    if isinstance(cfg, str):
        cfg = cfg.encode("utf-8")
    return frontend._parse_mapping_toml(BytesIO(cfg))


def test_toml_mapping_multiple_patterns():
    """
    Test that patterns may be specified as a list in TOML,
    and are expanded to multiple entries in the method map.
    """
    method_map, options_map = parse_toml("""
[[mappings]]
method = "python"
pattern = ["xyz/**.py", "foo/**.py"]
""")
    assert method_map == [
        ('xyz/**.py', 'python'),
        ('foo/**.py', 'python'),
    ]


@pytest.mark.parametrize(
    ("keywords_val", "expected"),
    [
        pytest.param('"foo bar quz"', {'bar': None, 'foo': None, 'quz': None}, id='string'),
        pytest.param('["foo", "bar", "quz"]', {'bar': None, 'foo': None, 'quz': None}, id='list'),
        pytest.param('"foo:1,2 bar quz"', {'bar': None, 'foo': (1, 2), 'quz': None}, id='s-args'),
        pytest.param('["bar", "foo:1,2", "quz"]', {'bar': None, 'foo': (1, 2), 'quz': None}, id='l-args'),
        pytest.param('[]', None, id='empty'),
    ],
)
def test_toml_mapping_keywords_parsing(keywords_val, expected):
    method_map, options_map = parse_toml(f"""
[[mappings]]
method = "python"
pattern = ["**.py"]
keywords = {keywords_val}
""")
    assert options_map['**.py'].get('keywords') == expected


@pytest.mark.parametrize(
    ("add_comments_val", "expected"),
    [
        ('"SPECIAL SAUCE"', ['SPECIAL SAUCE']),  # TOML will allow this as a single string
        ('["SPECIAL", "SAUCE"]', ['SPECIAL', 'SAUCE']),
        ('[]', None),
    ],
)
def test_toml_mapping_add_comments_parsing(add_comments_val, expected):
    method_map, options_map = parse_toml(f"""
[[mappings]]
method = "python"
pattern = ["**.py"]
add_comments = {add_comments_val}
""")
    assert options_map['**.py'].get('add_comments') == expected


@pytest.mark.parametrize("test_case", toml_test_cases_path.glob("bad.*.toml"), ids=lambda p: p.name)
def test_bad_toml_test_case(test_case: pathlib.Path):
    """
    Test that bad TOML files raise a ValueError.
    """
    with pytest.raises(frontend.ConfigurationError):
        with test_case.open("rb") as f:
            frontend._parse_mapping_toml(
                f,
                filename=test_case.name,
                style="pyproject.toml" if "pyproject" in test_case.name else "standalone",
            )



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from __future__ import annotations

import re
import shlex
from functools import partial
from io import BytesIO, StringIO

import pytest

from babel.messages import Catalog, extract, frontend
from babel.messages.frontend import (
    CommandLineInterface,
    ExtractMessages,
    UpdateCatalog,
)
from babel.messages.pofile import write_po
from tests.messages.consts import project_dir
from tests.messages.utils import CUSTOM_EXTRACTOR_COOKIE

mapping_cfg = """
[extractors]
custom = tests.messages.utils:custom_extractor

# Special extractor for a given Python file
[custom: special.py]
treat = delicious

# Python source files
[python: **.py]

# Genshi templates
[genshi: **/templates/**.html]
include_attrs =

[genshi: **/templates/**.txt]
template_class = genshi.template:TextTemplate
encoding = latin-1

# Some custom extractor
[custom: **/custom/*.*]
"""

mapping_toml = """
[extractors]
custom = "tests.messages.utils:custom_extractor"

# Special extractor for a given Python file
[[mappings]]
method = "custom"
pattern = "special.py"
treat = "delightful"

# Python source files
[[mappings]]
method = "python"
pattern = "**.py"

# Genshi templates
[[mappings]]
method = "genshi"
pattern = "**/templates/**.html"
include_attrs = ""

[[mappings]]
method = "genshi"
pattern = "**/templates/**.txt"
template_class = "genshi.template:TextTemplate"
encoding = "latin-1"

# Some custom extractor
[[mappings]]
method = "custom"
pattern = "**/custom/*.*"
"""


@pytest.mark.parametrize(
    ("data", "parser", "preprocess", "is_toml"),
    [
        (
            mapping_cfg,
            frontend.parse_mapping_cfg,
            None,
            False,
        ),
        (
            mapping_toml,
            frontend._parse_mapping_toml,
            None,
            True,
        ),
        (
            mapping_toml,
            partial(frontend._parse_mapping_toml, style="pyproject.toml"),
            lambda s: re.sub(r"^(\[+)", r"\1tool.babel.", s, flags=re.MULTILINE),
            True,
        ),
    ],
    ids=("cfg", "toml", "pyproject-toml"),
)
def test_parse_mapping(data: str, parser, preprocess, is_toml):
    if preprocess:
        data = preprocess(data)
    if is_toml:
        buf = BytesIO(data.encode())
    else:
        buf = StringIO(data)

    method_map, options_map = parser(buf)
    assert len(method_map) == 5

    assert method_map[1] == ('**.py', 'python')
    assert options_map['**.py'] == {}
    assert method_map[2] == ('**/templates/**.html', 'genshi')
    assert options_map['**/templates/**.html']['include_attrs'] == ''
    assert method_map[3] == ('**/templates/**.txt', 'genshi')
    assert (options_map['**/templates/**.txt']['template_class']
            == 'genshi.template:TextTemplate')
    assert options_map['**/templates/**.txt']['encoding'] == 'latin-1'
    assert method_map[4] == ('**/custom/*.*', 'tests.messages.utils:custom_extractor')
    assert options_map['**/custom/*.*'] == {}


def test_parse_keywords():
    kw = frontend.parse_keywords(['_', 'dgettext:2', 'dngettext:2,3', 'pgettext:1c,2'])
    assert kw == {
        '_': None,
        'dgettext': (2,),
        'dngettext': (2, 3),
        'pgettext': ((1, 'c'), 2),
    }


def test_parse_keywords_with_t():
    kw = frontend.parse_keywords(['_:1', '_:2,2t', '_:2c,3,3t'])

    assert kw == {
        '_': {
            None: (1,),
            2: (2,),
            3: ((2, 'c'), 3),
        },
    }


def test_extract_messages_with_t():
    content = rb"""
_("1 arg, arg 1")
_("2 args, arg 1", "2 args, arg 2")
_("3 args, arg 1", "3 args, arg 2", "3 args, arg 3")
_("4 args, arg 1", "4 args, arg 2", "4 args, arg 3", "4 args, arg 4")
"""
    kw = frontend.parse_keywords(['_:1', '_:2,2t', '_:2c,3,3t'])
    result = list(extract.extract("python", BytesIO(content), kw))
    expected = [(2, '1 arg, arg 1', [], None),
                (3, '2 args, arg 1', [], None),
                (3, '2 args, arg 2', [], None),
                (4, '3 args, arg 1', [], None),
                (4, '3 args, arg 3', [], '3 args, arg 2'),
                (5, '4 args, arg 1', [], None)]
    assert result == expected


def configure_cli_command(cmdline: str | list[str]):
    """
    Helper to configure a command class, but not run it just yet.

    :param cmdline: The command line (sans the executable name)
    :return: Command instance
    """
    args = shlex.split(cmdline) if isinstance(cmdline, str) else list(cmdline)
    cli = CommandLineInterface()
    cmdinst = cli._configure_command(cmdname=args[0], argv=args[1:])
    return cmdinst


@pytest.mark.parametrize("split", (False, True))
@pytest.mark.parametrize("arg_name", ("-k", "--keyword", "--keywords"))
def test_extract_keyword_args_384(split, arg_name):
    # This is a regression test for https://github.com/python-babel/babel/issues/384
    # and it also tests that the rest of the forgotten aliases/shorthands implied by
    # https://github.com/python-babel/babel/issues/390 are re-remembered (or rather
    # that the mechanism for remembering them again works).

    kwarg_specs = [
        "gettext_noop",
        "gettext_lazy",
        "ngettext_lazy:1,2",
        "ugettext_noop",
        "ugettext_lazy",
        "ungettext_lazy:1,2",
        "pgettext_lazy:1c,2",
        "npgettext_lazy:1c,2,3",
    ]

    if split:  # Generate a command line with multiple -ks
        kwarg_text = " ".join(f"{arg_name} {kwarg_spec}" for kwarg_spec in kwarg_specs)
    else:  # Generate a single space-separated -k
        specs = ' '.join(kwarg_specs)
        kwarg_text = f'{arg_name} "{specs}"'

    # (Both of those invocation styles should be equivalent, so there is no parametrization from here on out)

    cmdinst = configure_cli_command(
        f"extract -F babel-django.cfg --add-comments Translators: -o django232.pot {kwarg_text} .",
    )
    assert isinstance(cmdinst, ExtractMessages)
    assert set(cmdinst.keywords.keys()) == {'_', 'dgettext', 'dngettext',
                                            'dnpgettext', 'dpgettext',
                                            'gettext', 'gettext_lazy',
                                            'gettext_noop', 'N_', 'ngettext',
                                            'ngettext_lazy', 'npgettext',
                                            'npgettext_lazy', 'pgettext',
                                            'pgettext_lazy', 'ugettext',
                                            'ugettext_lazy', 'ugettext_noop',
                                            'ungettext', 'ungettext_lazy'}


def test_update_catalog_boolean_args():
    cmdinst = configure_cli_command(
        "update --init-missing --no-wrap -N --ignore-obsolete --previous -i foo -o foo -l en")
    assert isinstance(cmdinst, UpdateCatalog)
    assert cmdinst.init_missing is True
    assert cmdinst.no_wrap is True
    assert cmdinst.no_fuzzy_matching is True
    assert cmdinst.ignore_obsolete is True
    assert cmdinst.previous is False  # Mutually exclusive with no_fuzzy_matching


def test_compile_catalog_dir(tmp_path):
    """
    Test that `compile` can compile all locales in a directory.
    """
    locales = ("fi_FI", "sv_SE")
    for locale in locales:
        l_dir = tmp_path / locale / "LC_MESSAGES"
        l_dir.mkdir(parents=True)
        po_file = l_dir / 'messages.po'
        po_file.write_text('msgid "foo"\nmsgstr "bar"\n')
    cmdinst = configure_cli_command([  # fmt: skip
        'compile',
        '--statistics',
        '--use-fuzzy',
        '-d', str(tmp_path),
    ])
    assert not cmdinst.run()
    for locale in locales:
        assert (tmp_path / locale / "LC_MESSAGES" / "messages.mo").exists()


def test_compile_catalog_explicit(tmp_path):
    """
    Test that `compile` can explicitly compile a single catalog.
    """
    po_file = tmp_path / 'temp.po'
    po_file.write_text('msgid "foo"\nmsgstr "bar"\n')
    mo_file = tmp_path / 'temp.mo'
    cmdinst = configure_cli_command([  # fmt: skip
        'compile',
        '--statistics',
        '--use-fuzzy',
        '-i', str(po_file),
        '-o', str(mo_file),
        '-l', 'fi_FI',
    ])
    assert not cmdinst.run()
    assert mo_file.exists()


@pytest.mark.parametrize("explicit_locale", (None, 'fi_FI'), ids=("implicit", "explicit"))
def test_update_dir(tmp_path, explicit_locale: bool):
    """
    Test that `update` can deal with directories too.
    """
    template = Catalog()
    template.add("1")
    template.add("2")
    template.add("3")
    tmpl_file = tmp_path / 'temp-template.pot'
    with tmpl_file.open("wb") as outfp:
        write_po(outfp, template)
    locales = ("fi_FI", "sv_SE")
    for locale in locales:
        l_dir = tmp_path / locale / "LC_MESSAGES"
        l_dir.mkdir(parents=True)
        po_file = l_dir / 'messages.po'
        po_file.touch()
    cmdinst = configure_cli_command([  # fmt: skip
        'update',
        '-i', str(tmpl_file),
        '-d', str(tmp_path),
        *(['-l', explicit_locale] if explicit_locale else []),
    ])
    assert not cmdinst.run()
    for locale in locales:
        if explicit_locale and locale != explicit_locale:
            continue
        assert (tmp_path / locale / "LC_MESSAGES" / "messages.po").stat().st_size > 0


def test_extract_cli_knows_dash_s():
    # This is a regression test for https://github.com/python-babel/babel/issues/390
    cmdinst = configure_cli_command("extract -s -o foo babel")
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.strip_comments


def test_extract_cli_knows_dash_dash_last_dash_translator():
    cmdinst = configure_cli_command('extract --last-translator "FULL NAME EMAIL@ADDRESS" -o foo babel')
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.last_translator == "FULL NAME EMAIL@ADDRESS"


def test_extract_add_location():
    cmdinst = configure_cli_command("extract -o foo babel --add-location full")
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.add_location == 'full'
    assert not cmdinst.no_location
    assert cmdinst.include_lineno

    cmdinst = configure_cli_command("extract -o foo babel --add-location file")
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.add_location == 'file'
    assert not cmdinst.no_location
    assert not cmdinst.include_lineno

    cmdinst = configure_cli_command("extract -o foo babel --add-location never")
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.add_location == 'never'
    assert cmdinst.no_location


def test_extract_error_code(monkeypatch, capsys):
    monkeypatch.chdir(project_dir)
    cmdinst = configure_cli_command("compile --domain=messages --directory i18n --locale fi_BUGGY")
    assert cmdinst.run() == 1
    out, err = capsys.readouterr()
    if err:
        assert "unknown named placeholder 'merkki'" in err


@pytest.mark.parametrize("with_underscore_ignore", (False, True))
def test_extract_ignore_dirs(monkeypatch, capsys, tmp_path, with_underscore_ignore):
    pot_file = tmp_path / 'temp.pot'
    monkeypatch.chdir(project_dir)
    cmd = f"extract . -o '{pot_file}' --ignore-dirs '*ignored* .*' "
    if with_underscore_ignore:
        # This also tests that multiple arguments are supported.
        cmd += "--ignore-dirs '_*'"
    cmdinst = configure_cli_command(cmd)
    assert isinstance(cmdinst, ExtractMessages)
    assert cmdinst.directory_filter
    cmdinst.run()
    pot_content = pot_file.read_text()

    # The `ignored` directory is now actually ignored:
    assert 'this_wont_normally_be_here' not in pot_content

    # Since we manually set a filter, the otherwise `_hidden` directory is walked into,
    # unless we opt in to ignore it again
    assert ('ssshhh....' in pot_content) != with_underscore_ignore
    assert ('_hidden_by_default' in pot_content) != with_underscore_ignore


def test_extract_header_comment(monkeypatch, tmp_path):
    pot_file = tmp_path / 'temp.pot'
    monkeypatch.chdir(project_dir)
    cmdinst = configure_cli_command(f"extract . -o '{pot_file}' --header-comment 'Boing' ")
    cmdinst.run()
    pot_content = pot_file.read_text()
    assert 'Boing' in pot_content


@pytest.mark.parametrize("mapping_format", ("toml", "cfg"))
def test_pr_1121(tmp_path, monkeypatch, caplog, mapping_format):
    """
    Test that extraction uses the first matching method and options,
    instead of the first matching method and last matching options.

    Without the fix in PR #1121, this test would fail,
    since the `custom_extractor` isn't passed a delicious treat via
    the configuration.
    """
    if mapping_format == "cfg":
        mapping_file = (tmp_path / "mapping.cfg")
        mapping_file.write_text(mapping_cfg)
    else:
        mapping_file = (tmp_path / "mapping.toml")
        mapping_file.write_text(mapping_toml)
    (tmp_path / "special.py").write_text("# this file is special")
    pot_path = (tmp_path / "output.pot")
    monkeypatch.chdir(tmp_path)
    cmdinst = configure_cli_command(f"extract . -o {shlex.quote(str(pot_path))} --mapping {shlex.quote(mapping_file.name)}")
    assert isinstance(cmdinst, ExtractMessages)
    cmdinst.run()
    # If the custom extractor didn't run, we wouldn't see the cookie in there.
    assert CUSTOM_EXTRACTOR_COOKIE in pot_path.read_text()



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import pytest

from babel import plural

N_NODE = plural.ident_node('n')


def make_range_list(*values):
    ranges = []
    for v in values:
        if isinstance(v, int):
            val_node = plural.value_node(v)
            ranges.append((val_node, val_node))
        else:
            assert isinstance(v, tuple)
            ranges.append((plural.value_node(v[0]), plural.value_node(v[1])))
    return plural.range_list_node(ranges)


def n_eq(v):
    return 'relation', ('in', N_NODE, make_range_list(v))


def test_error_when_unexpected_end():
    with pytest.raises(plural.RuleError):
        plural._Parser('n =')


def test_eq_relation():
    assert plural._Parser('n = 1').ast == n_eq(1)


def test_in_range_relation():
    assert plural._Parser('n = 2..4').ast == (
        'relation',
        ('in', N_NODE, make_range_list((2, 4))),
    )


def test_negate():
    assert plural._Parser('n != 1').ast == plural.negate(n_eq(1))


def test_or():
    assert plural._Parser('n = 1 or n = 2').ast == ('or', (n_eq(1), n_eq(2)))


def test_and():
    assert plural._Parser('n = 1 and n = 2').ast == ('and', (n_eq(1), n_eq(2)))


def test_or_and():
    assert plural._Parser('n = 0 or n != 1 and n % 100 = 1..19').ast == (
        'or',
        (
            n_eq(0),
            (
                'and',
                (
                    plural.negate(n_eq(1)),
                    (
                        'relation',
                        (
                            'in',
                            ('mod', (N_NODE, plural.value_node(100))),
                            (make_range_list((1, 19))),
                        ),
                    ),
                ),
            ),
        ),
    )



from myproject.i18n import lazy_gettext as _l, lazy_ngettext as _n


class Choices:
    # SPECIAL: This comment should be extracted
    CHOICE_X = 1, _l("Choice X")
    # SPECIAL: Another special comment
    CHOICE_Y = 2, _l("Choice Y")
    # No comment...
    OPTION_C = 3, _l("Option C")
    # Test for _n too! (but no comment... shush...)
    OPTION_A = 4, (_n("Option A", "Options of the A kind", 1))



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from __future__ import annotations

import time
from datetime import datetime

import pytest
from freezegun import freeze_time

from babel import __version__ as VERSION
from babel.dates import format_datetime
from babel.messages import frontend
from babel.messages.frontend import OptionError
from babel.messages.pofile import read_po
from babel.util import LOCALTZ
from tests.messages.consts import TEST_PROJECT_DISTRIBUTION_DATA, data_dir, this_dir
from tests.messages.utils import Distribution


@pytest.fixture()
def extract_cmd(monkeypatch):
    monkeypatch.chdir(data_dir)
    dist = Distribution(TEST_PROJECT_DISTRIBUTION_DATA)
    extract_cmd = frontend.ExtractMessages(dist)
    extract_cmd.initialize_options()
    return extract_cmd


def test_neither_default_nor_custom_keywords(extract_cmd):
    extract_cmd.output_file = 'dummy'
    extract_cmd.no_default_keywords = True
    with pytest.raises(OptionError):
        extract_cmd.finalize_options()


def test_no_output_file_specified(extract_cmd):
    with pytest.raises(OptionError):
        extract_cmd.finalize_options()


def test_both_sort_output_and_sort_by_file(extract_cmd):
    extract_cmd.output_file = 'dummy'
    extract_cmd.sort_output = True
    extract_cmd.sort_by_file = True
    with pytest.raises(OptionError):
        extract_cmd.finalize_options()


def test_invalid_file_or_dir_input_path(extract_cmd):
    extract_cmd.input_paths = 'nonexistent_path'
    extract_cmd.output_file = 'dummy'
    with pytest.raises(OptionError):
        extract_cmd.finalize_options()


def test_input_paths_is_treated_as_list(extract_cmd, pot_file):
    extract_cmd.input_paths = data_dir
    extract_cmd.output_file = pot_file
    extract_cmd.finalize_options()
    extract_cmd.run()

    with pot_file.open() as f:
        catalog = read_po(f)
    msg = catalog.get('bar')
    assert len(msg.locations) == 1
    assert 'file1.py' in msg.locations[0][0]


def test_input_paths_handle_spaces_after_comma(extract_cmd, pot_file):
    extract_cmd.input_paths = f"{this_dir},  {data_dir}"
    extract_cmd.output_file = pot_file
    extract_cmd.finalize_options()
    assert extract_cmd.input_paths == [this_dir, data_dir]


def test_input_dirs_is_alias_for_input_paths(extract_cmd, pot_file):
    extract_cmd.input_dirs = this_dir
    extract_cmd.output_file = pot_file
    extract_cmd.finalize_options()
    # Gets listified in `finalize_options`:
    assert extract_cmd.input_paths == [extract_cmd.input_dirs]


def test_input_dirs_is_mutually_exclusive_with_input_paths(extract_cmd, pot_file):
    extract_cmd.input_dirs = this_dir
    extract_cmd.input_paths = this_dir
    extract_cmd.output_file = pot_file
    with pytest.raises(OptionError):
        extract_cmd.finalize_options()


@freeze_time("1994-11-11")
def test_extraction_with_default_mapping(extract_cmd, pot_file):
    extract_cmd.copyright_holder = 'FooBar, Inc.'
    extract_cmd.msgid_bugs_address = 'bugs.address@email.tld'
    extract_cmd.output_file = pot_file
    extract_cmd.add_comments = 'TRANSLATOR:,TRANSLATORS:'

    extract_cmd.finalize_options()
    extract_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. TRANSLATOR: This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

#: project/ignored/this_wont_normally_be_here.py:11
msgid "FooBar"
msgid_plural "FooBars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


@freeze_time("1994-11-11")
def test_extraction_with_mapping_file(extract_cmd, pot_file):
    extract_cmd.copyright_holder = 'FooBar, Inc.'
    extract_cmd.msgid_bugs_address = 'bugs.address@email.tld'
    extract_cmd.mapping_file = 'mapping.cfg'
    extract_cmd.output_file = pot_file
    extract_cmd.add_comments = 'TRANSLATOR:,TRANSLATORS:'

    extract_cmd.finalize_options()
    extract_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. TRANSLATOR: This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


@freeze_time("1994-11-11")
@pytest.mark.parametrize("ignore_pattern", ['**/ignored/**.*', 'ignored'])
def test_extraction_with_mapping_dict(extract_cmd, pot_file, ignore_pattern):
    extract_cmd.distribution.message_extractors = {
        'project': [
            (ignore_pattern, 'ignore', None),
            ('**.py', 'python', None),
        ],
    }
    extract_cmd.copyright_holder = 'FooBar, Inc.'
    extract_cmd.msgid_bugs_address = 'bugs.address@email.tld'
    extract_cmd.output_file = pot_file
    extract_cmd.add_comments = 'TRANSLATOR:,TRANSLATORS:'

    extract_cmd.finalize_options()
    extract_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Translations template for TestProject.
# Copyright (C) {time.strftime('%Y')} FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, {time.strftime('%Y')}.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: {date}\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. TRANSLATOR: This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


def test_extraction_add_location_file(extract_cmd, pot_file):
    extract_cmd.distribution.message_extractors = {
        'project': [
            ('**/ignored/**.*', 'ignore', None),
            ('**.py', 'python', None),
        ],
    }
    extract_cmd.output_file = pot_file
    extract_cmd.add_location = 'file'
    extract_cmd.omit_header = True

    extract_cmd.finalize_options()
    extract_cmd.run()

    expected_content = r"""#: project/file1.py
msgid "bar"
msgstr ""

#: project/file2.py
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    assert expected_content == pot_file.read_text()


def test_extraction_with_mapping_file_with_keywords(extract_cmd, pot_file):
    """
    Test that keywords specified in mapping config file are properly parsed,
    and merged with default keywords.
    """
    extract_cmd.mapping_file = 'mapping_with_keywords.cfg'
    extract_cmd.output_file = pot_file
    extract_cmd.input_paths = 'project'

    extract_cmd.finalize_options()
    extract_cmd.run()

    with pot_file.open() as f:
        catalog = read_po(f)

    for msgid in ('bar', 'Choice X', 'Choice Y', 'Option C', 'Option A'):
        msg = catalog[msgid]
        assert not msg.auto_comments  # This configuration didn't specify SPECIAL:...
        assert msg.pluralizable == (msgid == 'Option A')


def test_extraction_with_mapping_file_with_comments(extract_cmd, pot_file):
    """
    Test that add_comments specified in mapping config file are properly parsed.
    Uses TOML format to test that code path.
    """
    extract_cmd.mapping_file = 'mapping_with_keywords_and_comments.toml'
    extract_cmd.output_file = pot_file
    extract_cmd.input_paths = 'project/issue_1224_test.py'

    extract_cmd.finalize_options()
    extract_cmd.run()

    with pot_file.open() as f:
        catalog = read_po(f)

    # Check that messages were extracted and have the expected auto_comments
    for msgid, expected_comment in [
        ('Choice X', 'extracted'),
        ('Choice Y', 'special'),
        ('Option C', None),
        ('Option A', None),
    ]:
        msg = catalog[msgid]
        if expected_comment:
            assert any('SPECIAL' in comment and expected_comment in comment for comment in msg.auto_comments)
        else:
            assert not msg.auto_comments
        assert msg.pluralizable == (msgid == 'Option A')



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

from __future__ import annotations

import os
import shutil
from datetime import datetime

import pytest
from freezegun import freeze_time

from babel import __version__ as VERSION
from babel.dates import format_datetime
from babel.messages import frontend
from babel.util import LOCALTZ
from tests.messages.consts import (
    TEST_PROJECT_DISTRIBUTION_DATA,
    data_dir,
    get_po_file_path,
    i18n_dir,
)
from tests.messages.utils import Distribution


@pytest.fixture
def init_cmd(monkeypatch):
    monkeypatch.chdir(data_dir)
    dist = Distribution(TEST_PROJECT_DISTRIBUTION_DATA)
    init_cmd = frontend.InitCatalog(dist)
    init_cmd.initialize_options()
    yield init_cmd
    for dirname in ['en_US', 'ja_JP', 'lv_LV']:
        locale_dir = os.path.join(i18n_dir, dirname)
        if os.path.isdir(locale_dir):
            shutil.rmtree(locale_dir)


def test_no_input_file(init_cmd):
    init_cmd.locale = 'en_US'
    init_cmd.output_file = 'dummy'
    with pytest.raises(frontend.OptionError):
        init_cmd.finalize_options()


def test_no_locale(init_cmd):
    init_cmd.input_file = 'dummy'
    init_cmd.output_file = 'dummy'
    with pytest.raises(frontend.OptionError):
        init_cmd.finalize_options()


@freeze_time("1994-11-11")
def test_with_output_dir(init_cmd):
    init_cmd.input_file = 'project/i18n/messages.pot'
    init_cmd.locale = 'en_US'
    init_cmd.output_dir = 'project/i18n'

    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# English (United States) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: en_US\n"
"Language-Team: en_US <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    with open(get_po_file_path('en_US')) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_keeps_catalog_non_fuzzy(init_cmd):
    init_cmd.input_file = 'project/i18n/messages_non_fuzzy.pot'
    init_cmd.locale = 'en_US'
    init_cmd.output_dir = 'project/i18n'

    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# English (United States) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: en_US\n"
"Language-Team: en_US <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    with open(get_po_file_path('en_US')) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_correct_init_more_than_2_plurals(init_cmd):
    init_cmd.input_file = 'project/i18n/messages.pot'
    init_cmd.locale = 'lv_LV'
    init_cmd.output_dir = 'project/i18n'

    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en')
    expected_content = fr"""# Latvian (Latvia) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: lv_LV\n"
"Language-Team: lv_LV <LL@li.org>\n"
"Plural-Forms: nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 :"
" 2);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""

"""
    with open(get_po_file_path('lv_LV')) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_correct_init_singular_plural_forms(init_cmd):
    init_cmd.input_file = 'project/i18n/messages.pot'
    init_cmd.locale = 'ja_JP'
    init_cmd.output_dir = 'project/i18n'

    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='ja_JP')
    expected_content = fr"""# Japanese (Japan) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: ja_JP\n"
"Language-Team: ja_JP <LL@li.org>\n"
"Plural-Forms: nplurals=1; plural=0;\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""

"""
    with open(get_po_file_path('ja_JP')) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_supports_no_wrap(init_cmd):
    init_cmd.input_file = 'project/i18n/long_messages.pot'
    init_cmd.locale = 'en_US'
    init_cmd.output_dir = 'project/i18n'

    long_message = '"' + 'xxxxx ' * 15 + '"'

    with open('project/i18n/messages.pot', 'rb') as f:
        pot_contents = f.read().decode('latin-1')
    pot_with_very_long_line = pot_contents.replace('"bar"', long_message)
    with open(init_cmd.input_file, 'wb') as f:
        f.write(pot_with_very_long_line.encode('latin-1'))
    init_cmd.no_wrap = True

    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en_US')
    expected_content = fr"""# English (United States) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: en_US\n"
"Language-Team: en_US <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid {long_message}
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    with open(get_po_file_path('en_US')) as f:
        actual_content = f.read()
    assert expected_content == actual_content


@freeze_time("1994-11-11")
def test_supports_width(init_cmd):
    init_cmd.input_file = 'project/i18n/long_messages.pot'
    init_cmd.locale = 'en_US'
    init_cmd.output_dir = 'project/i18n'

    long_message = '"' + 'xxxxx ' * 15 + '"'

    with open('project/i18n/messages.pot', 'rb') as f:
        pot_contents = f.read().decode('latin-1')
    pot_with_very_long_line = pot_contents.replace('"bar"', long_message)
    with open(init_cmd.input_file, 'wb') as f:
        f.write(pot_with_very_long_line.encode('latin-1'))
    init_cmd.width = 120
    init_cmd.finalize_options()
    init_cmd.run()

    date = format_datetime(datetime(1994, 11, 11, 00, 00), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale='en_US')
    expected_content = fr"""# English (United States) translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language: en_US\n"
"Language-Team: en_US <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator coment,
#. that will include several lines
#: project/file1.py:8
msgid {long_message}
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

"""
    with open(get_po_file_path('en_US')) as f:
        actual_content = f.read()
    assert expected_content == actual_content



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import os
from io import BytesIO

from babel.messages import Catalog, mofile
from babel.support import Translations

data_dir = os.path.join(os.path.dirname(__file__), 'data')


def test_basics():
    mo_path = os.path.join(data_dir, 'project', 'i18n', 'de',
                           'LC_MESSAGES', 'messages.mo')
    with open(mo_path, 'rb') as mo_file:
        catalog = mofile.read_mo(mo_file)
        assert len(catalog) == 2
        assert catalog.project == 'TestProject'
        assert catalog.version == '0.1'
        assert catalog['bar'].string == 'Stange'
        assert catalog['foobar'].string == ['Fuhstange', 'Fuhstangen']



def test_sorting():
    # Ensure the header is sorted to the first entry so that its charset
    # can be applied to all subsequent messages by GNUTranslations
    # (ensuring all messages are safely converted to unicode)
    catalog = Catalog(locale='en_US')
    catalog.add('', '''\
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n''')
    catalog.add('foo', 'Voh')
    catalog.add(('There is', 'There are'), ('Es gibt', 'Es gibt'))
    catalog.add('Fizz', '')
    catalog.add(('Fuzz', 'Fuzzes'), ('', ''))
    buf = BytesIO()
    mofile.write_mo(buf, catalog)
    buf.seek(0)
    translations = Translations(fp=buf)
    assert translations.ugettext('foo') == 'Voh'
    assert translations.ungettext('There is', 'There are', 1) == 'Es gibt'
    assert translations.ugettext('Fizz') == 'Fizz'
    assert translations.ugettext('Fuzz') == 'Fuzz'
    assert translations.ugettext('Fuzzes') == 'Fuzzes'


def test_more_plural_forms():
    catalog2 = Catalog(locale='ru_RU')
    catalog2.add(('Fuzz', 'Fuzzes'), ('', '', ''))
    buf = BytesIO()
    mofile.write_mo(buf, catalog2)


def test_empty_translation_with_fallback():
    catalog1 = Catalog(locale='fr_FR')
    catalog1.add('', '''\
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n''')
    catalog1.add('Fuzz', '')
    buf1 = BytesIO()
    mofile.write_mo(buf1, catalog1)
    buf1.seek(0)
    catalog2 = Catalog(locale='fr')
    catalog2.add('', '''\
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n''')
    catalog2.add('Fuzz', 'Flou')
    buf2 = BytesIO()
    mofile.write_mo(buf2, catalog2)
    buf2.seek(0)

    translations = Translations(fp=buf1)
    translations.add_fallback(Translations(fp=buf2))

    assert translations.ugettext('Fuzz') == 'Flou'



import subprocess
import sys

from setuptools import Command, setup

try:
    from babel import __version__
except SyntaxError as exc:
    sys.stderr.write(f"Unable to import Babel ({exc}). Are you running a supported version of Python?\n")
    sys.exit(1)


class import_cldr(Command):
    description = 'imports and converts the CLDR data'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        subprocess.check_call([sys.executable, 'scripts/download_import_cldr.py'])


setup(
    name='babel',
    version=__version__,
    description='Internationalization utilities',
    long_description='A collection of tools for internationalizing Python applications.',
    author='Armin Ronacher',
    author_email='armin.ronacher@active-4.com',
    maintainer='Aarni Koskela',
    maintainer_email='akx@iki.fi',
    license='BSD-3-Clause',
    url='https://babel.pocoo.org/',
    project_urls={
        'Source': 'https://github.com/python-babel/babel',
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Internationalization',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.8',
    packages=['babel', 'babel.messages', 'babel.localtime'],
    package_data={"babel": ["py.typed"]},
    include_package_data=True,
    install_requires=[
        # This version identifier is currently necessary as
        # pytz otherwise does not install on pip 1.4 or
        # higher.
        # Python 3.9 and later include zoneinfo which replaces pytz
        'pytz>=2015.7; python_version<"3.9"',
    ],
    extras_require={
        'dev': [
            "tzdata;sys_platform == 'win32'",
            'backports.zoneinfo; python_version<"3.9"',
            'freezegun~=1.0',
            'jinja2>=3.0',
            'pytest-cov',
            'pytest>=6.0',
            'pytz',
            'setuptools',
        ],
    },
    cmdclass={'import_cldr': import_cldr},
    zip_safe=False,
    # Note when adding extractors: builtin extractors we also want to
    # work if packages are not installed to simplify testing.  If you
    # add an extractor here also manually add it to the "extract"
    # function in babel.messages.extract.
    entry_points="""
    [console_scripts]
    pybabel = babel.messages.frontend:main

    [distutils.commands]
    compile_catalog = babel.messages.setuptools_frontend:compile_catalog
    extract_messages = babel.messages.setuptools_frontend:extract_messages
    init_catalog = babel.messages.setuptools_frontend:init_catalog
    update_catalog = babel.messages.setuptools_frontend:update_catalog

    [distutils.setup_keywords]
    message_extractors = babel.messages.setuptools_frontend:check_message_extractors

    [babel.checkers]
    num_plurals = babel.messages.checkers:num_plurals
    python_format = babel.messages.checkers:python_format

    [babel.extractors]
    ignore = babel.messages.extract:extract_nothing
    python = babel.messages.extract:extract_python
    javascript = babel.messages.extract:extract_javascript
    """,
)
